import java.io.IOException;
import java.io.OutputStream;
import javax.net.ssl.HttpsURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.lang.reflect.*;

public class PUT_Vehicle
{
  private Vehicle _vehicle;
  
  public void PutVehicle(String authString, String putVehicleUri, String appId)
  {   
    try
    {      
      // Construct authentication header for FIM   
      String authHeader = String.format("Atmosphere atmosphere_app_id=%s, Bearer %s", appId, authString);             
      
      // Serialize vehicle object into JSON for PUT request body     
      String json = DummySerializer(_vehicle);           
      
      URL url = new URL(putVehicleUri);                                        // Create URL object from uri parameter   
      HttpsURLConnection conn = (HttpsURLConnection) url.openConnection();     // Begin new Https connection
      conn.setRequestMethod("PUT");                                            // Set REST method to PUT  
      conn.setRequestProperty("Authorization", authHeader);                    // Set Authorization Header   
      conn.setRequestProperty("Content-Type", "application/json");             // Set Content-Type to json 
      conn.setDoOutput(true);                                                  // Allow output to come back in response 
      
      OutputStream os = conn.getOutputStream();   // Create new output stream for body from connection   
      os.write(json.getBytes());                  // Transmit the json workorder object       
      os.close();                                 // Flush and close stream   

      // Read response, 204 "no content" evaluation for "success", if not throw exception 
      if (conn.getResponseCode() != HttpsURLConnection.HTTP_NO_CONTENT) {
        throw new RuntimeException("Failed : HTTP error code : " + conn.getResponseCode());
      }
      else{
        System.out.println("Vehicle Updated Successfully!");
        _vehicle = null;                         // Clear object
      }
     
      conn.disconnect();                         // End connection to free resources
       
    } catch (MalformedURLException e) {          // Required Exception
      e.printStackTrace();
    } catch (IOException e) {                    // Required Exception
      e.printStackTrace();
    }
  } 

  // Public method used to create the Vehicle object which contains. 
  // After serialization, this will be used as the body for the PUT command
  public void CreateNewPutVehicle(String name, String number, String registrationNumber, String vin, String make, String model,
                                  Integer year, Double tankCapacity, Double highwayMpg, Double cityMpg, Integer vehicleSize)
  {
    _vehicle = new Vehicle();
    _vehicle.Name = name;
    _vehicle.VehicleNumber = number;
    _vehicle.RegistrationNumber = registrationNumber;
    _vehicle.VIN = vin;
    _vehicle.Make = make;
    _vehicle.Model = model;
    _vehicle.Year = year;
    _vehicle.TankCapacity = tankCapacity;
    _vehicle.HighwayMPG = highwayMpg;
    _vehicle.CityMPG = cityMpg;
    _vehicle.VehicleSize = vehicleSize;
  }
  
  // Serializer - "object to JSON" for Vehicle object
  private String DummySerializer(Object object)
  {    
    try{
      StringBuilder sb = new StringBuilder();
      sb.append("{");                                           // Open parentheses
      
      for (Field field : object.getClass().getDeclaredFields()) // Loop through object
      {
        field.setAccessible(true);                              // Override
        if (field != null && field.getName() != "this$0" )      // Skip "this$0" - outer class
        {
          sb.append("\"" + field.getName() + "\" : ");
          sb.append(field.get(object) != null ? ("\"" + field.get(object) + "\"") : "null");   
          sb.append(", ");
        }
      }
      
      sb.setLength(sb.length() - 2);                            // Remove last ", "
      sb.append("}");                                           // Close parentheses
      
      return sb.toString();
      
    } catch (IllegalAccessException e) {                        // Required exception
      throw new RuntimeException("Error accessing object value!");
    }
  }
  
  public class Vehicle
  {
    public String VehicleNumber;
    public String Name;
    public String RegistrationNumber;
    public String VIN;
    public String Make;
    public Integer Year;
    public String Model;
    public Double TankCapacity;
    public Double HighwayMPG;
    public Double CityMPG;
    public Integer VehicleSize;
  }  
}